#!/usr/local/bin/perl
#
# Integrated Permutation Testing script for the randomization of a list of genes and their comparing with a full POSOC clustering output.
# Four command line parameters are required -- Firstly, the number of iterations of randomization required should be supplied as a command line argument. The default number of iterations is 100. This should be followed by the three required file names: The list of all genes Affy IDs; the list of differentially expressed Affy IDs, and the POSOC clustering output.
# Ian Grieve
# 21/07/05

use strict;
use warnings;
use List::Util 'shuffle';

# Part 1: Create a randomized subset of genes from the supplied genes list. The default size of this subset list is 280 genes, but this can be changed.

my $full_genes_list = "";
my $iterations_requested = "";
my $de_genes_count = "";
my $posoc_all_clusters = "";
my $cluster_min_genes = 0;

if ($#ARGV < 3)
{
    die print "Please supply the command line arguments required.\nFour command line arguments should be supplied.\n";
}


if ($ARGV[0] < 1)
{
    $iterations_requested = 100;
}

else
{
    $iterations_requested = $ARGV[0];
    $full_genes_list = $ARGV[1];
    $de_genes_count = $ARGV[2];
    $posoc_all_clusters = $ARGV[3];
    $cluster_min_genes = $ARGV[4];
}

print "Iterations Requested: ";
print $iterations_requested;
print "\n";

open (FULL_LIST, $full_genes_list) || die print "Cannot open $full_genes_list in Random Genes List Generator";
my @genes_list = <FULL_LIST>;

open (DE_GENES_COUNT, $de_genes_count) || die print "Permutation Test Script cannot open $de_genes_count";
my @de_genes_count = <DE_GENES_COUNT>;

if ($de_genes_count[0] !~ /_[a|s]t/)
    {
	shift (@de_genes_count);
    }

my $random_geneslist_length = $#de_genes_count + 1;
print $random_geneslist_length;
print "\n";

sub randomize
{
our $randomized_subset = 'randomized_subset.txt';
open (RAND_OUTPUT, ">$randomized_subset") || die print "Cannot open $randomized_subset in Random Genes List Generator";

my @genes_list_shuffled = shuffle (@genes_list);

for (my $i=0; $i<=$random_geneslist_length; $i++)
{
    chomp $genes_list_shuffled[$i];
    print RAND_OUTPUT $genes_list_shuffled[$i];
    print RAND_OUTPUT "\n";
}
}

close (DE_GENES_COUNT);
randomize;


# Part 2: The matching of the randomized genes list with the clusters is carried out in order to generate an output delineating the numbers of matched genes for each of the clusters in the POSOC output and listing these.

our $randomized_list = 'randomized_subset.txt';
open (RANDOMIZED_LIST, $randomized_list) || die print "Permutation Test Script cannot open $randomized_list";
my @randomized_list = <RANDOMIZED_LIST>;
my $cluster_count = 0;
my $match_count = 0;

open (POSOC_ALLCLUSTERS, $posoc_all_clusters) || die print "Permutation Test Script cannot open $posoc_all_clusters";

my @posoc_allclusters_tosplit = <POSOC_ALLCLUSTERS>;
my $posoc_allclusters_tosplit = "@posoc_allclusters_tosplit";
my @posoc_allclusters = split /-{45}/, $posoc_allclusters_tosplit;

my $posoc_allcluster_genes = $posoc_allclusters[2];
my @posoc_allcluster_genes = split /Cluster\s/, $posoc_allcluster_genes;

shift @posoc_allcluster_genes;
our $posoc_allclusters_total = $#posoc_allcluster_genes;
print $posoc_allclusters_total;
print "\n";
our $iterations_total = $posoc_allclusters_total + 1;
print $iterations_total;

my $permoutput = "permutations.txt";
open (PERMUTATION_OUTPUT, ">>$permoutput") || die print "Permutation Test Script cannot open $permoutput";

sub match_genes
{
    $cluster_count = 0;
    open (RANDOMIZED_LIST, $randomized_list) || die print "Permutation Test Script cannot open $randomized_list";
    my @randomized_list = <RANDOMIZED_LIST>;
    print $randomized_list;
    print "\n";
    print @randomized_list;
    print "\n";
    foreach my $cluster_geneslist (@posoc_allcluster_genes)
    {
	$cluster_count = $cluster_count + 1;
	print PERMUTATION_OUTPUT "CLUSTER ";
	print PERMUTATION_OUTPUT $cluster_count;
#	print PERMUTATION_OUTPUT "\n";
	$match_count = 0;
	
	foreach my $randomized_gene (@randomized_list)
	{
	    chomp $randomized_gene;
	    $randomized_gene =~ /[^=]*/;
#	    print $randomized_gene;
#	    print "\n";
	    
	    if ($cluster_geneslist =~ /$randomized_gene/)
	    {
#		print PERMUTATION_OUTPUT "Match!\t";
#		print PERMUTATION_OUTPUT $randomized_gene;
#		print PERMUTATION_OUTPUT "\n";
		$match_count = $match_count + 1;
	    }
	}
	
	print PERMUTATION_OUTPUT ": ";
	print PERMUTATION_OUTPUT $match_count;
	print PERMUTATION_OUTPUT "\n\n";   
    }
}

for (my $iteration = 0; $iteration < $iterations_requested; $iteration++)
# Change 100 to a different figure for more or fewer random permutations.
{
    randomize;
    close (RAND_OUTPUT);
    close (FULL_LIST);
    print PERMUTATION_OUTPUT "\n\n## Iteration ";
    print PERMUTATION_OUTPUT $iteration + 1;
    print PERMUTATION_OUTPUT " ##\n\n";
    match_genes;
}

close (RAND_OUTPUT);
close (FULL_LIST);
close (RANDOMIZED_LIST);
close (PERMUTATION_OUTPUT);

sub de_match
{
    my $de_list_outfile = 'de_outfile_temp.txt';
    my $de_list_file = $ARGV[2];
    $cluster_count = 0;
    $match_count = 0;

    open (DE_LIST_PROBEID_ONLY, ">$de_list_outfile") || die print "Permutation Test Script cannot open $de_list_outfile";
    open (DE_LIST_FILE, $de_list_file) || die print "Permutation Test Script cannot open $de_list_file";

    my @de_list_split = <DE_LIST_FILE>;
    if ($de_list_split[0] !~ /_[a|s]t/)
    {
	shift (@de_list_split);
    }
    
    foreach my $de_list_probeid (@de_list_split)
    {
	if ($de_list_probeid =~ /([^\s]*_[a|s]t)/)
	{
	    print DE_LIST_PROBEID_ONLY $1;
	    print DE_LIST_PROBEID_ONLY "\n";
	}
    }

    close (DE_LIST_PROBEID_ONLY);
 
    our $randomized_list = 'de_outfile_temp.txt';

    my $de_counts_output = "de_counts.txt";
    open (PERMUTATION_OUTPUT, ">>$de_counts_output") || die print "Permutation Test Script cannot open $permoutput";
    
    match_genes;
}

de_match;

close (RANDOMIZED_LIST);
close (POSOC_ALLCLUSTERS);
close (PERMUTATION_OUTPUT);
unlink 'de_outfile_temp.txt';


# Part 3: Vectors are created for each cluster, and compared with the total differentially expressed genes for that cluster (in de_counts). The probability that the number of differentially expressed genes in the cluster is significantly less than or greater than the number of randomly chosen genes is found. If this probability (in either direction) is less than 0.05, this is indicated in the output.

#our $vector_counts_debug = 'vectordebug.txt';
#open (VECTOR_COUNTS_DEBUG, ">>$vector_counts_debug") || die print "Permutation Test Script Part 3 cannot open $vector_counts_debug";

our $vector_counts_out = 'vectorcounts.txt';
open (VECTOR_COUNTS, ">>$vector_counts_out") || die print "Cannot open $vector_counts_out in Permutation Test Script Part 3";

our $perminput = 'permutations.txt';
open (PERMUTATION_INPUT, $perminput) || die print "Permutation Test Script Part 3 cannot open $perminput";

my @perminput = <PERMUTATION_INPUT>;
my $perminput_split_byiteration = "@perminput";
my @perminput_split_byiteration = split /## Iteration/, $perminput_split_byiteration;

shift @perminput_split_byiteration;
my $de_to_compare = "";
my $iteration = 1;
my $cluster_number_toprint = 0;
my $clusterlessthancount = 0;
my $equalgenesno = 0;
my $clustergreaterthancount = 0;
my $perm_iteration_genesno = 0;
# my @clustervector = "";

sub vector_maker
{
    print VECTOR_COUNTS "\n";
    print VECTOR_COUNTS "Cluster ";
    print VECTOR_COUNTS $iteration;
    print VECTOR_COUNTS ": ";
    foreach my $perm_iteration (@perminput_split_byiteration)
    {
#	print VECTOR_COUNTS_DEBUG "\n\n";
	if ($perm_iteration =~ /CLUSTER $iteration:\s+(\d+)/g)
	{
	    $perm_iteration_genesno = $1;
	    print VECTOR_COUNTS $1;
#	    push (@clustervector, $1);
	    print VECTOR_COUNTS ", ";
	}

	else {next;}
	
    }
#    print VECTOR_COUNTS_DEBUG @clustervector;
#    print VECTOR_COUNTS_DEBUG "NEXT***CLUSTER";
#    print VECTOR_COUNTS_DEBUG "\n";
    
}

vector_maker;

while ($iteration <= $iterations_total)
# If there are more 'all genes' clusters, this number should be increased.
{
$iteration = $iteration + 1;
print VECTOR_COUNTS "\n\n";
vector_maker;
}

close (VECTOR_COUNTS);
close (PERMUTATION_INPUT);
close (DE_INPUT);
unlink 'permutations.txt';
unlink 'randomized_subset.txt';

our $vector_counts_in = 'vectorcounts.txt';
open (VECTOR_COUNTS_IN, $vector_counts_in) || die print "Permutation Test Script could not open $vector_counts_in";

our $de_input = 'de_counts.txt';
open (DE_INPUT, $de_input) || die print "Permutation Test Script Part 3 cannot open $de_input";
my @de_input = <DE_INPUT>;

my @vector_counts_input = <VECTOR_COUNTS_IN>;

our $permutation_tests_out = 'permutation_tests.txt';
open (PERMUTATION_TESTS, ">$permutation_tests_out") || die print "Cannot open $permutation_tests_out in Permutation Test Script, Part 3";

my $de_input_cluster_number = "";
my $de_input_cluster_genes = "";
my $vector_input_cluster_genes = "";
my @vector_input_cluster_genes_tosplit = "";
my $less_than_counter = 0;
my $greater_than_counter = 0;
my $cluster_counts_total = 0;
my $cluster_counts_adjusted_total = 0;
my $cluster_less_than_pvalue = 0;
my $cluster_greater_than_pvalue = 0;

foreach my $de_input_line (@de_input)
{
    
    if ($de_input_line =~ /CLUSTER (\d+): (\d+)/)
    {
	$de_input_cluster_number = $1;
	$de_input_cluster_genes = $2;
	
	if ($de_input_cluster_genes > $cluster_min_genes)
	{
	    print PERMUTATION_TESTS "\n\n";
	    print PERMUTATION_TESTS "##";
	    print PERMUTATION_TESTS "\n";
	    print PERMUTATION_TESTS $de_input_cluster_number;
	    print PERMUTATION_TESTS "\t";
	    print PERMUTATION_TESTS $de_input_cluster_genes;
	    print PERMUTATION_TESTS "\n\n";
	}
	next; next;
    }
    
foreach my $vector_counts_line (@vector_counts_input)
{
    if ($vector_counts_line =~ /Cluster $de_input_cluster_number:(.*)/)
    {
	$vector_input_cluster_genes = $1;
	@vector_input_cluster_genes_tosplit = split(/, /, $vector_input_cluster_genes);	
    
    foreach my $vector_genes_count (@vector_input_cluster_genes_tosplit)
    {
	if ($vector_genes_count =~ /(\d+)/)
	{
	    if ($1 < $de_input_cluster_genes)
	    {
		$less_than_counter = $less_than_counter + 1;
	    }

	    elsif ($1 > $de_input_cluster_genes)
	    {
		$greater_than_counter = $greater_than_counter + 1;
	    }

	    elsif ($1 == $de_input_cluster_genes)
	    {
		$less_than_counter = $less_than_counter + 1;
		$greater_than_counter = $greater_than_counter + 1;
	    }
	    
	    else 
	    {
		print PERMUTATION_TESTS "Error!";
	    }
	    
	}
    }
	
	$cluster_counts_total = $less_than_counter + $greater_than_counter;
	
	if ($cluster_counts_total > 0)
	{
	    $cluster_less_than_pvalue = $less_than_counter / $cluster_counts_total;
	}
	else 
	{
	    $cluster_less_than_pvalue = 1;
	}
	
	if ($cluster_counts_total > 0)
	{
	    $cluster_greater_than_pvalue = $greater_than_counter / $cluster_counts_total;
	}
	else 
	{
	    $cluster_greater_than_pvalue = 1;
	}
	
	if ($de_input_cluster_genes > $cluster_min_genes)
	{
	    print PERMUTATION_TESTS "Less-Than P-value for Cluster ";
	    print PERMUTATION_TESTS $de_input_cluster_number;
	    print PERMUTATION_TESTS ": ";
#	print PERMUTATION_TESTS $cluster_less_than_pvalue;
	    printf PERMUTATION_TESTS ("%.5f", $cluster_less_than_pvalue);
	    print PERMUTATION_TESTS "\n";
	    print PERMUTATION_TESTS "Greater-Than P-value for Cluster ";
	    print PERMUTATION_TESTS $de_input_cluster_number;
	    print PERMUTATION_TESTS ": ";
#	print PERMUTATION_TESTS $cluster_greater_than_pvalue;
	    printf PERMUTATION_TESTS ("%.5f", $cluster_greater_than_pvalue);
	    print PERMUTATION_TESTS "\n";
	}
	
	if ($cluster_less_than_pvalue < 0.05 && $de_input_cluster_genes > $cluster_min_genes)
	{
	    print PERMUTATION_TESTS "The number of DE genes in Cluster ";
	    print PERMUTATION_TESTS $de_input_cluster_number;
	    print PERMUTATION_TESTS " is significantly fewer than expected";
	    print PERMUTATION_TESTS "\n\n";
	}

	elsif ($cluster_greater_than_pvalue < 0.05 && $de_input_cluster_genes > $cluster_min_genes)
	{
	    print PERMUTATION_TESTS "The number of DE genes in Cluster ";
	    print PERMUTATION_TESTS $de_input_cluster_number;
	    print PERMUTATION_TESTS " is significantly greater than expected";
	    print PERMUTATION_TESTS "\n\n";
	}

	elsif ($de_input_cluster_genes > $cluster_min_genes) 
	{
	    print PERMUTATION_TESTS "\n\n";
	}

	$less_than_counter = 0;
	$greater_than_counter = 0;
	
    }
}

}

close (VECTOR_COUNTS_IN);
close (DE_INPUT);
close (PERMUTATION_TESTS);

unlink 'vectorcounts.txt';
unlink 'de_counts.txt';
# If the above two lines are deleted or commented out the script will produce superfluous output the next time it is run.

# Use of uninitialized value in numeric gt (>) at permutation_test.pl line 386 <VECTOR_COUNTS_IN> line 776. (also line 302 and line 370). Warning, though script seems to be working.



